import asyncio
import logging
from typing import List, Optional, Dict

import pandas as pd
from komoutils.core import KomoBase, safe_ensure_future

from aporacle.data.asset import AssetData
from aporacle.data.gcp import download_csv_from_gcp_return_df
from aporacle.data.symbols import SymbolData


class Combination(KomoBase):
    def __init__(self, feed: str, symbols: List[str] = None, source: Optional[str] = "gcp"):
        self.feed: str = str(feed).replace('/', '_')
        self.symbols: Optional[list] = symbols
        self.source: str = source
        self.collected: Dict[str, pd.DataFrame] = {}
        self.combined: Optional[pd.DataFrame] = None

    def combine(self, data: List[pd.DataFrame]):
        self.combined = pd.concat(data, axis=1).sort_index(axis=0)
        return self.combined

    async def symbol_data_collection_task(self, symbol: str):
        if self.source == "gcp":
            df = download_csv_from_gcp_return_df(bucket_name=self.feed, file_name=symbol)
        else:
            raise Exception("Unsupported data source specified. ")

        prefix = f'{symbol}_x_'
        df.columns = [prefix + col for col in df.columns]  # Modify columns in-place
        self.collected[symbol] = df

    async def collect_symbol_data(self):
        self.collected = {}
        await asyncio.gather(*[self.symbol_data_collection_task(symbol) for symbol in self.symbols])

    def get(self):
        asyncio.run(self.collect_symbol_data())
        return self.combine(data=list(self.collected.values()))



import asyncio
import json
import logging
from typing import Optional

import websockets
from komoutils.core import KomoBase, safe_ensure_future
from komoutils.core.time import the_time_in_iso_now_is

from aporacle import conf
from aporacle.conf import WS, streams


class DataBroadcaster(KomoBase):
    def __init__(self, stream: str = 'utilities'):
        self.stream: str = stream
        self.url: str = f"{conf.WS}{streams[stream]}/ingest"
        self.broadcast_stream: asyncio.Queue = asyncio.Queue()
        self._broadcast_data_stream_task: Optional[asyncio.Task] = None

    @property
    def name(self):
        return "data_broadcaster"

    def start(self):
        self._broadcast_data_stream_task = safe_ensure_future(self._broadcast_data_stream_loop())

    async def _broadcast_data_stream_loop(self):
        self.log_with_clock(log_level=logging.INFO, msg=f"Establishing a connection to {self.url}. ")
        try:
            async with websockets.connect(self.url) as websocket:
                self.log_with_clock(log_level=logging.INFO, msg=f"Connection to {self.url} for {self.stream} data "
                                                                f"is now active. ")
                while True:
                    message = await self.broadcast_stream.get()
                    message['time_created'] = the_time_in_iso_now_is()
                    await websocket.send(json.dumps(message))
                    self.broadcast_stream.task_done()

        except Exception as e:
            self.log_with_clock(log_level=logging.ERROR, msg=f"Connection was closed. Will retry {e}")
            await asyncio.sleep(5)
            safe_ensure_future(self._broadcast_data_stream_loop())



import asyncio
from typing import List, Optional

import pandas as pd
from komoutils.core import KomoBase

from aporacle.data.gcp import download_csv_from_gcp_return_df


class FeedResults(KomoBase):
    def __init__(self, feed: str, source: Optional[str] = "gcp"):
        self.feed: str = str(feed).replace('/', '_')
        self.source: str = source
        self.collected: Optional[pd.DataFrame] = None

    async def data_collection_task(self):
        if self.source == "gcp":
            df = download_csv_from_gcp_return_df(bucket_name=self.feed, file_name=f'{self.feed}_results'.lower())
        else:
            raise Exception("Unsupported data source specified. ")

        self.collected = df

    def get(self):
        asyncio.run(self.data_collection_task())
        return self.collected
